"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.guard = void 0;
const symbols_1 = require("./symbols");
const utils_1 = require("./utils");
function guard(name, fn, handleOptions, defaultOptions = {}) {
    function getFnWithErrors(options) {
        (0, utils_1.assert)(fn);
        return (value, path = []) => {
            const valid = fn(value, options);
            if (!valid)
                return {
                    valid,
                    errors: [
                        {
                            message: `vality.${name}.base`,
                            path,
                            options,
                            value,
                        },
                    ],
                };
            if (handleOptions === undefined)
                return { valid: true, value, errors: [] };
            const keysWithError = Object.keys(options).filter(k => handleOptions[k] !== undefined && !handleOptions[k](value, options[k], options));
            return {
                valid: keysWithError.length === 0,
                errors: keysWithError.map(k => ({
                    message: `vality.${name}.options.${k}`,
                    options,
                    path,
                    value,
                })),
            };
        };
    }
    return Object.assign((options) => {
        return {
            [symbols_1._validate]: getFnWithErrors(Object.assign(Object.assign({}, defaultOptions), options)),
            [symbols_1._options]: undefined,
            [symbols_1._type]: undefined,
        };
    }, {
        [symbols_1._validate]: getFnWithErrors(defaultOptions),
        [symbols_1._options]: undefined,
        [symbols_1._type]: undefined,
    });
}
exports.guard = guard;
