"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("./utils");
const valit_1 = require("./valit");
const vality_1 = require("./vality");
vality_1.vality.array = (0, valit_1.valit)("array", e => (value, path, options) => {
    if (!Array.isArray(value))
        return { valid: false, errors: [{ message: "vality.array.base", path, options, value }] };
    const fn = (0, utils_1.enyToGuardFn)(e);
    const errors = [];
    for (const k in value) {
        // We can do this assertion here, since in the worst case, we'll get undefined, which is what we want to
        const res = fn(value[k], [...path, k]);
        errors.push(...res.errors);
        if (!res.valid && options.bail)
            break;
    }
    return { valid: errors.length === 0, errors };
}, {
    minLength: (val, o) => val.length >= o,
    maxLength: (val, o) => val.length <= o,
    bail: val => true,
});
vality_1.vality.object = (0, valit_1.valit)("object", e => (value, path, options) => {
    if (typeof value !== "object" || value === null)
        return { valid: false, errors: [{ message: "vality.object.base", path, options, value }] };
    const errors = [];
    for (const k in e) {
        // We can do this assertion here, since in the worst case, we'll get undefined, which is what we want to
        const res = (0, utils_1.enyToGuardFn)(e[k])(value[k], [...path, k]);
        errors.push(...res.errors);
        if (!res.valid && options.bail)
            break;
    }
    return { valid: errors.length === 0, errors };
});
vality_1.vality.optional = (0, valit_1.valit)("optional", e => (val, path) => {
    if (val === undefined)
        return { valid: true, errors: [] };
    return (0, utils_1.enyToGuardFn)(e)(val, path);
});
vality_1.vality.enum = (0, valit_1.valit)("enum", (...es) => (value, path, options) => {
    const valid = es.some(e => (0, utils_1.enyToGuardFn)(e)(value, path).valid);
    return { valid, errors: valid ? [] : [{ message: "vality.enum.base", path, options, value }] };
});
vality_1.vality.tuple = (0, valit_1.valit)("tuple", (...es) => (value, path, options) => {
    if (!Array.isArray(value) || value.length !== es.length)
        return { valid: false, errors: [{ message: "vality.tuple.base", path, options, value }] };
    const errors = (0, utils_1.flat)(value.map((_, i) => i).map(i => (0, utils_1.enyToGuardFn)(es[i])(value[i], [...path, i]).errors));
    return { valid: errors.length === 0, errors };
});
// Gotta parse here as this is the exception where we don't return Valit<E>, but actually VirtualValit<E>
vality_1.vality.virtual = (0, valit_1.valit)("virtual", utils_1.enyToGuardFn);
